// gmodstat_calls.h
//

//
// Routine call definitions
//

void parse_config_file(struct global_names *gn,
					   struct global_state *gs);

void parse_config_line(struct global_names *gn,
					   struct global_state *gs,
					   char *line);
					
void init_gmodstat_state(struct global_state *gs,
					struct global_names *gn);

//
//

int normalize_dname(struct dname_list **ptr);

void find_domainname_prefix(struct seen_playername *tmpplist,
							struct hostaddrlist *hlist);

void create_pname_suffix_stats(struct global_state *gs);

void find_common_prefix(struct seen_playername *plist);


// Routines for building/sorting list of AS numbers
// associated with players

void add_asnum_to_tree(struct asnum_tree **ptr,
				  struct asnumlist *entry);

void sort_asnum_tree(struct asnum_tree **ptr);

void dump_asnum_tree(struct global_state *gs);

void sort_as_prefix_tree(struct as_prefix_tree **ptr);

//
// Routines for creating game usage stats and graphs
//

void player_rankings1(struct global_state *gs);

void player_rankings2(struct global_state *gs);

void cumulative_gametime(struct global_state *gs);

void player_ping_vs_frags(struct global_state *gs,
							struct global_names *gn);
							
void player_ping_vs_gameratio(struct global_state *gs,
							struct global_names *gn);

void client_ping_vs_gametime(struct global_state *gs);

void client_ping_vs_gamenum(struct global_state *gs);

void median_client_pings(struct global_state *gs); 

void topN_client_pings(struct global_state *gs);

void topN_client_phistos(struct global_state *gs);

void find_ping_hilow(struct global_state *gs,
					 int *lping,
					 int *mping,
					 int *hping);

int new_find_ping_hilow(struct global_state *gs,
					 float *array,
					 float count,
					 int *lping,
					 int *mping,
					 int *hping);

void single_client_game_pings(struct global_state *gs);

void single_client_pings(struct global_state *gs);

void single_client_phisto(struct global_state *gs);

int is_right_client(struct seen_player *seenp,
					struct global_state *gs);

void create_ptime_histo(struct global_state *gs,
								struct global_names *gn);
								
void create_selected_ptime_histo(struct global_state *gs,
								struct global_names *gn);
								
void create_ptime_histo_onegame(int hour,
					  struct seen_games *glist,
					  struct global_state *gs);

void create_ping_histo(struct global_state *gs,
								struct global_names *gn);
								
void create_selected_ping_histo(struct global_state *gs,
								struct global_names *gn);
								
void create_ping_histo_onegame(int hour,
					  struct seen_games *glist,
					  struct global_state *gs);

void add_ping_to_histo(int newping,
					   float incr,
					  struct global_state *gs);

void new_add_ping_to_histo(int newping,
					   float incr,
					  struct global_state *gs,
					  float *array,
					  float *counter);

int extract_one_phisto(struct player_cphisto *cph,
						float *tmparray,
						int *low,
						int *mid,
						int *hi,
						struct global_state *gs);

int is_in_region(struct seen_player *plist,
				 struct global_state *gs);

int is_in_unresolved(struct seen_player *plist,
					 struct global_state *gs);

int is_name_in_dname(char *name, char *dname);

void print_game_stats(struct global_state *gs);

void print_player_stats(struct global_state *gs,
						struct seen_games *game,
						int ss,
						char *player);

void print_token_stats(struct global_state *gs,
						struct seen_cmdtoken *tlist,
						int ss);

void print_say_stats(struct global_state *gs,
					 struct seen_games *game,
					 int ss);

void print_ipaddr_stats(struct seen_playername *plist, int ss);


// Routines supporting the inverted tree of domain name
// fragments

void push_domain_name(struct global_state *gs,
					  struct dname_list **dlist,
					  char *name);

void dump_dname(struct dname_list *ptr,
				int tab,
				int total,
				int mtime,
				struct global_state *gs);

void sort_dname(struct dname_list **ptr);

int count_home_domains(struct dname_list *nametop,
						struct hostaddrlist *flist,
						int global_stats);

struct dname_list *find_dname_match(struct dname_list *nametop,
									char *name);

void list_all_domain_names(struct global_names *gn,
							struct global_state *gs);

// Routines supporting the initialization of the
// internal domain name mapping structures

int sanity_check(struct global_state *gs);

struct hostaddrlist *create_hlist(struct global_state *gs,
								FILE *rfile);

struct hostaddrlist *install_fragmentlist(struct global_state *gs,
											FILE *fragmentfile);

void add_to_hlist(struct global_state *gs,
					FILE *file,
					struct hostaddrlist **hlist);

void init_dns_lists(struct global_names *gn,
					struct global_state *gs);

void init_asnum_list(struct global_names *gn,
					struct global_state *gs);

void push_new_asnum(struct global_state *gs,
					unsigned int tmpaddr,
					int asnum,
					unsigned int prefix,
					int prefix_len,
					int multirec,
					int multiasnum,
					char *descr);


void do_whois(struct global_state *gs,
			    struct in_addr addr,
				int *asnum,
				unsigned int *prefix,
				int *prefix_len,
				int *multirec,
				int *multiasnum,
				char *descr);

void resolve_asnums(struct global_names *gn,
					struct global_state *gs);

int resolve_one_asnum(struct player_ipaddr *ipaddr,
					  struct global_state *gs);

void list_all_asnums(struct global_state *gs);

int hacked_fgets(char *buf,
				 int count,
				 SOCKET src);

int hacked_wouldnotblock(SOCKET src);


// Routines associated with building the
// global seen_client table

void add_newclients(struct seen_player *seenp,
					struct global_state *gs);

struct hostaddrlist *resolve_clientaddr(struct in_addr *iaddr,
										struct global_state *gs);

int push_new_clientports(struct seen_client *tmpclient,
					   struct seen_player *seenp,
					   struct global_state *gs);

int push_new_clientpings(struct seen_client *tmpclient,
					   struct seen_player *seenp,
					   struct global_state *gs);

void print_client_list(struct global_state *gs,
						struct global_names *gn);

void client_stats(int min_gametime,
				struct global_state *gs);

void sort_seen_client(struct global_state *gs);


// Routines associated with parsing the client
// events contained in the log file

int median_ping(struct global_state *gs,
				struct seen_player *tmpplayer);

int frag_count(struct seen_player *tmpplayer);

void weed_out_idleplayers(struct seen_games *gl,
						struct global_state *gs);

void sort_by_pings(struct seen_games *gl,
					struct global_state *gs);

void sort_by_frags(struct seen_games *gl,
					struct global_state *gs);

void CHECK_FRAG_COUNTS(struct global_state *gs);

void sort_items(struct seen_player *seenp);

int parse_time(char **p);

int parse_line(char *oneline,
			   int *ts,
			   struct global_state *gs);

struct seen_cmdtoken *parse_token(char **p,
								 int ts,
								 struct global_state *gs);

int parse_time2(struct global_state *gs,
				FILE *in_f);

int parse_line2(FILE *in_f,
			   int *ts,
			   struct global_state *gs);

struct seen_cmdtoken *parse_token2(FILE *in_f, int ts,
								  struct global_state *gs);

long walk_log_files(struct global_names *gn,
					struct global_state *gs);

int read_log_file(FILE *inf,
				  char *logfilename,
				  struct global_state *gs);

int extract_base_time(char *name,
					  struct global_state *gs);

void player_in(struct seen_player *tp,
               int ts,
               int playmode,
               struct global_state *gs);

void player_out(struct seen_player *tp,
				int ts);

void update_killedbyworld(struct seen_player *seenp,
					  int wp,
					  struct global_state *gs);

void update_kill_list(struct player_kill **klist,
					  struct seen_player *who,
					  int wp,
					  struct global_state *gs);

void c_add_ipaddr(struct seen_player *seenp,
				  struct in_addr *iaddr,
				  int port,
				  struct global_state *gs);

struct client_id *findclient_id(struct client_id *cidptr, int cidnum);

struct seen_games *initgame(char *p,
							int ts,
							struct seen_games **gl,
							struct global_state *gs);

struct client_id *clientconnect(char *p,
								int ts,
								struct global_state *gs);

struct client_id  *clientuserinfochanged(char *p,
								int ts,
								struct global_state *gs);

struct client_id *clientbegin(char *p,
								int ts,
								struct global_state *gs);

struct seen_player *clientdisconnect(char *p,
								int ts,
								struct global_state *gs);

struct client_id *clientkill(char *p,
								int ts,
								struct global_state *gs);

struct seen_games *gamescore(char *p,
							int ts,
							struct global_state *gs);

struct seen_games *exitgame(char *p,
							int ts,
							struct global_state *gs);

struct seen_games *shutdowngame(char *p,
								int ts,
								struct global_state *gs);

struct client_id *clientsay(char *p,
								int ts,
								struct global_state *gs,
								int saycid);

struct seen_cmdtoken *token_stats(struct global_state *gs,
								  char *token,
								  int tstamp,
								  struct seen_cmdtoken **tl);

void setbasetime(char *ptr,
				 int tstamp,
				 struct global_state *gs);

void setgmodversion(char *ptr,
				 int tstamp,
				 struct global_state *gs);

void clientping(char *ptr,
				 int tstamp,
				 struct global_state *gs);

void clientpinghisto(char *p,
				 int ts,
				 struct global_state *gs);

void clientitem(char *p,
				int ts,
				struct global_state *gs);

// Other stuff

void process_unix_ping(struct global_names *gn,
						struct global_state *gs);


