//
// gmodstat_defines.h
//
// Various constants and macros
// (and substitutions required to
// support both MS VisualC++ and gcc)
//

#define PACKAGE "gmodstat"
#define VERSION "0.2.1"

#define DEBUG(Z,X)	if ((int)debug >= Z) printf X
#define MSG_OUT(X)	{fprintf(global_outf, X) ; printf X;}

#define IS_ODD_PORT(X)  (((X) < 27960) || ((X) > 27963))

#define FLAG_SET(X,Y)	((X & Y) == Y)
#define FLAG_UNSET(X,Y)	((X & Y) == 0)

#define LOGDEFAULT  "serv.log\0"
#define OUTFDEFAULT	"logout.txt\0"
#define CONFDEFAULT	"gmodstatconf.txt\0"

#define MAXLINE  2048	// Max num of characters expected in a log line
#define TOKENLEN 64		// Max num of characters expected in a token


// Variations for non-Windows environment (this code originally written
// under MS VisualC++ 6.0). If variable WIN32 is not set, make some
// substitutions
#ifndef WIN32
#define SOCKET int
#define stricmp strcasecmp
#define WSAGetLastError()  -1
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#define closesocket(X) close(X)
#endif

// For "stats" variable

#define GAME_STATS		1	// If set, print game stats
#define PLAYER_STATS	2	// Set to print player stats
#define HUMAN_READABLE	4	// Set to print readable ASCII stats
#define C_TOKEN_STATS	8	// Set to print out list of seen commands
#define GAME_NAME		16	// Specify map name of specific game
#define GAME_TIME		32	// Specify time of specific game
#define PLAYER_NAME		64	// Specify player name to print out

#define OPEN_WHOIS		256	// Open connection to 'whois' RADB server
#define CLIENT_NAMES	512	// Print the seen_client list
#define NO_CLIENT_CALC	1024// Suppress creation of seen_client lists

#define UNNAMED_PLAYER	4096// Include UnnamedPlayer in aggregate stats
#define BASE_TIME		8192// Attempt to extract basetime from logfile name structure
#define MAX_GAMETIME	16384// Count clients < N minutes rather than > N minutes
#define CUMULATE_TIME	32768// Plot cumulative total played time vs day of year

#define HACK_STOP		65536// Tmp flag for internal use

// For graph_stats variable

#define GRAPH_PINGS		1	// Generate scatter/histos of median pings over many games
#define INCLUDE_REGIONS	2	// For graphs, only count players falling in 'home' domains
#define EXCLUDE_REGIONS	4	// For graphs, only count players NOT falling in 'home' domains
#define ONLY_UNRESOLVED	8	// For graphs, only count players with unresolved IP addresses
#define XGRAPH_OUT		16	// Include "Weeks N" lines for use by "xgraph -nl -m" plotting tool
#define PING_HISTO		32	// Create histograms (not scatter plots) of ClientPing: data (ping samples)
#define PHISTO_TOTAL	64	// Create aggregate histos (not distinct histos) of CPhisto2: data
#define GRAPH_MANY		128	// Create multiple ping files, one for each 'home domain'
#define PTIME_HISTO		256	// Create histo of played time vs (integer) hour of week
#define DAY_OF_YEAR		512	// For some scatter plots, use day of year not day of week
#define HOUR_OF_DAY		1024// For some scatter plots, use hour of day not day of week
#define PHISTO_GAME		2048// Create per-game aggregate histos from CPhisto2: data.
#define PHISTO_MEDIAN	4096// Create per-CPhisto2 time vs median ping
#define PHISTO_JITTER	8192// Create per-CPhisto2 time vs jitter
#define HALF_DAYS		16384// For cumulative played time, XY samples only every 12 hours

// For ping_stats variable

#define TOPN_CLIENT		2	// ClientPing ping times for topN seen_client entries
#define MEDIAN_PINGS	4	// Plot median vs jitter for seen_client ping times
#define PINGVSGAMENUM	8	// Plot median vs number of games in seen_client entries
#define PINGVSGAMETIME	16	// Plot median vs gamelengths in seen_client entries
#define PINGVSFRAGS		32	// Plot median ping vs frag count per game (seen_player)
#define DO_JITTER		64	// Modify PINGVSGAMENUM and PINGVSGAMETIME to plot jitter
#define PROCESS_NIXPING	128	// Special processing for unix ping text file, plot raw
							// pings and histogram
#define DO_DEVIATION	256	// For certain ping processing, plot jitter/median_ping

#define SINGLE_C_PHISTO	1024// Aggregate or per-game ping histograms for specified client
							// (playername+suffix) based on CPhisto2: data
#define SINGLE_C_PING	2048// Ping scatter/histograms for specified client (playername+suffix)
							// based on ClientPing: data
#define TOPN_PHISTOS	4096// CPhisto2 histograms for topN seen_client entries
#define PINGVSGRATIO	8192// Plot median ping vs playedtimeratio per game (seen_player)

// For dom_stats variable
#define FAKE_RESOLVE	1	// ipaddr that cannot be resolved become "w-x-y-z.unresolved"

#define RESOLVE_AGAIN	4	// Re-resolve domain names that previously failed resolution
#define DUMP_RESOLVES	8	// Print the results of IPaddr:name resolution process
#define DUMP_DTREE		16	// Print the inverted domain name tree
#define DUMP_DOMAINMAP	32	// After reading and resolving, dump ipaddr:domainname table
#define SUFFIX_STATS	64	// Analyse client distribution based on domain suffix matching

#define RESOLVE_ASNUM	512	// Resolve AS number of each seen ip address
#define DUMP_ASNUMMAP	1024// After reading and resolving, dump ipaddr->ASnum data
#define DUMP_ASTREE		2048// Dump AS num sorted in descending order of appearance
#define RESOLVE_MISSING	4096// Resolve names for ipaddr that don't exist in gs->dnslist
#define FAKE_RESOLVE2	8192// unresolvable ipaddr become "w-x-y-z.w-x-y-z.unresolved"
#define CHK_HLIST_UNIQ	16384// When reading in hlist entries from file, check for redundancy


// The following are for routines still under development
// for identifying the AS numbers of each client's IP address

#define WHOIS_PORT	43					// Default 'whois' port
#define WHOIS_ADDR	"198.108.0.18\0"	// Default whois host is whois.ra.net

#define BLOCK_TIME	30000	// Allow 30ms before concluding that the whois
							// server has completed its response to a specific ipaddr
