// gmodstat_stats.cpp
//
// Copyright (c) 2001, Grenville Armitage
// gj_armitage@yahoo.com
//
//
// +++++++++++++++++++++++ License and Conditions +++++++++++++++++++++++++
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   A copy of this license was supplied with gmodstat in the
//   file ../COPYING. Alternatively, a copy may be obtained from
//   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA  02111-1307, USA.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Routines for post-analysis of various pieces
// of information gathered from one or more
// Quake3 logfiles.
//

#include "stdafx.h"
#include "gmodstat.h"


//
// print_token_stats()
//

void print_token_stats(struct global_state *gs,
						struct seen_cmdtoken *tlist,
						int ss)
{
	int numtokens = 0, seentokens = 0;

	while (tlist != NULL)
	{
		if ((ss & HUMAN_READABLE) == HUMAN_READABLE)
		 fprintf(gs->g_outf, "> %s seen %d times, first time at %d.\n",
			 tlist->cmdtoken, tlist->numseen, tlist->tstamp);
		else
		 fprintf(gs->g_outf, "%s %d %d\n",
			 tlist->cmdtoken, tlist->numseen, tlist->tstamp);

		numtokens++;
		seentokens += tlist->numseen;
		
		tlist = tlist->next;
	}

	if ((ss & HUMAN_READABLE) == HUMAN_READABLE)
	 fprintf(gs->g_outf, "%d different tokens seen a total of %d times.\n",
		 numtokens, seentokens);
	else
	 fprintf(gs->g_outf, "%d %d\n",
		 numtokens, seentokens);

	return;
}



//
//	create_pname_suffix_stats()
//
// For a range of total playing time from 0 to gs->playtimemin
// dump the seen_clients list entries who played more than X minutes.
//
//

void create_pname_suffix_stats(struct global_state *gs)
{
	int n = 0;

	fprintf(gs->g_outf, "\nclient_stats:\n");
	
	if ((gs->g_outf3 = fopen("client_stats.txt\0", "w")) == NULL)
	{
		printf("create_pname_suffix_stats(): Couldn't open client_stats.txt for writing. Exiting.\n");
		fprintf(gs->g_outf, "create_pname_suffix_stats(): Couldn't open client_stats.txt for writing. Exiting.\n");
		exit(-1);
	}

	while (n <= gs->playtimemin)
	{
		printf("\rclient_stats for min_gametime %d   ",n/60);
		client_stats(n, gs);
		n += gs->rangeinc;
	}
	printf("\n");

	if (gs->g_outf3 != NULL)
		fclose(gs->g_outf3);
		
	return;
}


//
// print_game_stats()
//

void print_game_stats(struct global_state *gs)
{
	struct  seen_games *glist, *gtmp;
	int human;
	char *map = gs->mapname;
	int maptime = gs->maptime;
	char *player = gs->playername;

	// If game stats are required
	if (FLAG_SET(gs->stats, GAME_STATS) == 0)
		return;

	human = FLAG_SET(gs->stats, HUMAN_READABLE); // Flag

	// Games are stored last->first hanging off glist. Walk the
	// list until we find the last entry, then step backwards up the
	// list, printing stats as we go

	gtmp = gs->gamelist;
	while (gtmp != NULL)
	{
		glist = gtmp;
		gtmp = gtmp->next;
	}

	if (glist == NULL)	// gamelist pointed to an empty list anyway
		return;

	// otherwise, glist points to entry at end of list, chronologically
	// the first one parsed from the logfile

	while (glist != NULL)
	{
	 int gname, gtime, gmap, gmaptime;

	 gname = FLAG_SET(gs->stats, GAME_NAME); // Do we care about specific names?
	 gtime = FLAG_SET(gs->stats, GAME_TIME); // Or specific times?
	 gmap = (strcmp(glist->mapname, map) == 0); // Does the name match?
	 gmaptime = (glist->t_start == maptime);	// Does the time match?

	 if ((!gname && !gtime) || (gname && gmap) || (gmaptime && gtime))
	 {
		long starttime = glist->t_base + glist->t_start;
		char *timestr1 = ctime(&starttime);
		char *timestr2;

		timestr2 = strchr(timestr1, '\n');
		if (timestr2 != NULL)
			*timestr2 = 0;

		 fprintf(gs->g_outf, "%s %s ", timestr1, glist->mapname);

		 if (glist->next != NULL)
		 {
			 if (human)
				fprintf(gs->g_outf, "After %d ",
					(glist->t_start - (glist->next)->t_end));
			 else
				fprintf(gs->g_outf, "%d ",
					(glist->t_start - (glist->next)->t_end));
		 }
		 else
		 {
			 if (human)
				fprintf(gs->g_outf, "After 0 ");
			 else
				fprintf(gs->g_outf, "0 ");
		 }

		 if (human)
			fprintf(gs->g_outf, "Start %d:%02d (%d) Stop %d:%02d Len %d Names %d Players %d\n",
				 glist->t_start / 60, glist->t_start % 60, glist->t_start,
				 glist->t_end / 60, glist->t_end % 60,
				 (glist->t_end - glist->t_start),
				 glist->max_p_name,
				 glist->max_c_id + 1);
		 else
			fprintf(gs->g_outf, "%d %d %d %d %d\n",
				 glist->t_start,
				 glist->t_end,
				 (glist->t_end - glist->t_start),
				 glist->max_p_name,
				 glist->max_c_id + 1);
	
		 // Print this game's player stats (if ss flags are set)
		 print_player_stats(gs, glist, gs->stats, player);

		 // If "say" statements were recorded in this game,
		 // print them out

		 if (gs->saycid >= 0)
			 print_say_stats(gs, glist, gs->stats);
	 }

	 glist = glist->prev;
	 //glist = NULL;
	}

	return;
}


//
// print_player_stats()
//

void print_player_stats(struct global_state *gs,
						struct seen_games *game,
						int ss,
						char *player)
{
	struct	seen_player *plist;
	struct player_inout *pio;
	struct player_kill *pk;
	struct player_item *items;
	int human;

	// If game stats are required
	plist = NULL;
	if (FLAG_SET(ss, PLAYER_STATS))
		plist = game->c_all;

	human = ((ss & HUMAN_READABLE) == HUMAN_READABLE); // Flag

	while (plist != NULL)
	{		
	 if ((strcmp(plist->playername, player) == 0) || FLAG_UNSET(ss, PLAYER_NAME))
	 {
		 fprintf(gs->g_outf, "  %s seen %d namechanges %d",
			 plist->playername, plist->numseen, plist->namechanged);
 		 fprintf(gs->g_outf, " endfrags %d endping %d cphcnt %d pingcnt %d items %d totaltime %d ",
			 plist->frags, plist->ping, plist->cphcnt, plist->pingcnt, plist->itemcnt, plist->totaltime);
		 if (plist->c_addr != NULL)
			 fprintf(gs->g_outf, "(IP: %s)\n", inet_ntoa(plist->c_addr->iaddr));
		 else
			 fprintf(gs->g_outf, "(bot)\n");


		 // List the in/out times
		 pio = plist->inout;
		 while (pio != NULL)
		 {
			 fprintf(gs->g_outf, "    Joined %d:%02d Left %d:%02d Frags %d (ROUGHLY)\n",
				 pio->in_ts / 60, pio->in_ts % 60,
				 pio->out_ts / 60, pio->out_ts % 60,
				 pio->frags);
			 pio = pio->next;
		 }

		 // List the kills
		 pk = plist->kill;
		 while (pk != NULL)
		 {
			 fprintf(gs->g_outf, "     Killed %s  %d times with %s\n",
				 pk->who, pk->count, weapname[pk->how]);
			 pk = pk->next;
		 }
		 // List the deaths

		 pk = plist->death;
		 while (pk != NULL)
		 {
			 if (pk->who != NULL)
				 fprintf(gs->g_outf, "     Killed by %s  %d times with %s\n",
					pk->who, pk->count, weapname[pk->how]);
			 else
				 fprintf(gs->g_outf, "     Killed by <WORLD>  %d times with %s\n",
					pk->count, weapname[pk->how]);

			 pk = pk->next;
		 }

		 // List the items

		 items = plist->items;
		 while (items != NULL)
		 {
			 fprintf(gs->g_outf, "     Item: %s  %d times\n",
				 items->itemname, items->itemcnt);
			 items = items->next;
		 }
	 }

		 // Move to next player
		 plist = plist->next;
	}

	return;
}

//
// print_say_stats()
//

void print_say_stats(struct global_state *gs,
					 struct seen_games *game,
					 int ss)
{
	struct	seen_say *say;

	if ((say = game->say) == NULL)	// Nothing was 'said' during this game
		return;
	
	// Find end of the say list
	while (say->next != NULL)
		say = say->next;

	// Walk backwards along the say list
	while (say != NULL)
	{		
		fprintf(gs->g_outf, "  At %d:%02d %s said %s\n",
			(say->tstamp)/60, (say->tstamp)%60,
			say->who->playername, say->saytext);

		 // Move to next statement recorded
		 say = say->prev;
	}

	return;
}






// cumulative_gametime()
//
// Walk through the seen_game list and plot the cumulative
// total played time vs day (and fractional day) of year.
//

void cumulative_gametime(struct global_state *gs)
{
	struct  seen_games *glist;
	struct	seen_player *plist;
	long totalplayedtime;
	float weektime;
	float prev_weektime;
	long realtime;
	struct tm *tmnow;
	char outfname[64];

	// Games are stored last->first hanging off glist. Walk the
	// list until we find the last entry, then step backwards up the
	// list, printing stats as we go

	if ((glist = gs->gamelist) == NULL)
		return;

	while (glist->next != NULL)
		glist = glist->next;

	// glist points to entry at end of list, chronologically
	// the first one parsed from the logfile

	// Establish new output file
	sprintf(outfname,"cumulativetime.txt\0");
	if ((gs->g_outf2 = fopen(outfname, "w")) == NULL)
	{
		printf("cumulative_gametime(): Opening output file %s failed, aborting.\n",
			outfname);
		exit(1);
	}
    		
	printf("Generating cumulative gametime to file %s\n",
				outfname);
	fprintf(gs->g_outf, "Generating cumulative gametime to file %s\n",
				outfname);

	totalplayedtime = 0;
	prev_weektime = 0.0;

	while (glist != NULL)
	{
		plist = glist->c_all;
		while (plist != NULL)
		{
			if (plist->c_addr != NULL)
				totalplayedtime += plist->totaltime;

			plist = plist->next;
		}

		realtime = glist->t_base + glist->t_end;
		tmnow = localtime(&realtime);
		weektime = (tmnow->tm_hour / (float)24) +
			(tmnow->tm_min / (float)1440) +
			(tmnow->tm_sec / (float)86400);

		// If DAY_OF_YEAR flag is set, the x-axis is
		// time of day of the year, otherwise time of
		// day of the week. The former provides a linear
		// sense of the player's overall trend, the latter
		// shows any weekly repeating patterns

		if (FLAG_SET(gs->graph_stats, DAY_OF_YEAR))
			weektime += tmnow->tm_yday;
		else
			weektime += tmnow->tm_wday;

		if (FLAG_SET(gs->graph_stats, HALF_DAYS) && FLAG_SET(gs->graph_stats, DAY_OF_YEAR))
		{
			if ((weektime - prev_weektime) >= 0.5)
			{
				prev_weektime = weektime;
				fprintf(gs->g_outf2, "%8.6f , %8.4f\n",
					weektime , totalplayedtime/(float)86400);
			}		
		}
		else
			fprintf(gs->g_outf2, "%8.6f , %8.4f\n",
				weektime , totalplayedtime/(float)86400);

		glist = glist->prev;
	}
	
	if (gs->g_outf2 != NULL)
		fclose(gs->g_outf2);

	return;
}

