// gmodstat_structs.h
//
// All the key structures
//
//

// Global parameters such as flags, file names, pointers, etc are
// stored in a large global_state struct, and in most cases I
// simply pass a pointer to the current global_stat struct between
// routines.

struct global_names {

	char	configfile[128];		// Name of the file from which to read configs
	char	logfile[128];			// Name of the source log file
	char	logfile2[128];			// Name of the 2nd source log file
	char	logfile2_base[128];		// Base string for names in 2nd source log file
	char	resolvfile[128];		// Name of ipaddr:name resolver file
	char	namereresolvfile[128];	// File containing 'dig' updates to resolvfile
	char	namefragmentfile[128];	// Name of host domain fragment file
	char	domainlimitfile[128];	// File listing the only top level domains to count
	char	outfile[128];			// Prefix of name to use to generate output files
	char	global_outfile[128];	// Outfile+"0.txt", intial stats file
	char	asnumfile[128];			// Name of the ipaddr->ASnumber mapping file
	char	whoisaddr[128];			// IP address of 'whois' RADB to query
	char	unixpingfile[128];		// Name of file containing unix ping results for parsing
};

struct global_state {

	FILE *g_outf;	// File descriptor for global_outfile file
	FILE *g_outf2;		// Temp file descriptor
	FILE *g_outf3;		// Temp file descriptor

	long time_base;		// Current logfile's base start date expressed in
						// seconds since midnight Jan.1st 1970
	int time_offset;	// Add this value (+/- in hours) to whatever basetime
						// is extracted from logfile (or logfile name) when
						// calculating 'time_base' parameter (default: 0)

	SOCKET whoishost;	// Socket for connection to the whois RADB host

	// Flags controlling behavior of various routines
	int		stats;		// Flags controlling general activities
	int		dom_stats;	// Flags controlling domain name related activities
	int		graph_stats;// Flags controlling graph output related activities
	int		ping_stats;	// Flags controlling analysis relating to ping times
	float	gmod_ver;	// What version of 'gmod' was running on the QIII server
						// to generate CPhisto2, etc, lines. (0, 0.2, 0.8, 1.0, 1.1)
	int		saycid;		// If this is > -1 then capture "say" lines
	int		clienttime;	// For topN_client_pings, this is the min playedtime
	float	itemratio;	// If a player picks up less than 'itemratio' items
						// per minute it is considered to be idle/irrelevant.

	int		playtimemin;	// Run the suffix analysis multiple times, excluding players
							// who played less than X minutes, for 0 < X < playtimemin
	int		rangeinc;		// For the above two analyses, increment the playtime limit
							// by rangeinc (default is 240 seconds)

	// Parameters for faking domain suffixes when creating seen_client entries
	// for unresolved ipaddr "w.x.y.z"

	int		faked_suffix;	// 1, faked suffix is ".w.unresolved"
							// 2, faked suffix is ".w-x.unresolved"
							// 3, faked suffix is ".w-x-y.unresolved"
							// 4, faked suffix is ".w-x-y-z.unresolved" (default)

	// Parameters to restrict gamestats printouts
	char	mapname[TOKENLEN];		// Print games with map named mapname[]
	char	playername[TOKENLEN];	// Print games with player named playername[]
	int		maptime;				// Print game starting at maptime (or -1)
	char	playersuffix[TOKENLEN];	// Look for player with this addr suffix

	// Parameters to control various ping and playedtime analyses
	float	playedratio_min;		// Min. played time to game time ratio for graph outputs
	int		max_ping;				// Maximum valid client ping time (for histos, etc)
	int		ping_step;				// When doing ping histograms, bucket size is 'ping_step'
	float	ping_total;				// Normalized total ping values added to the histogram
	float	*ping_histo;			// Pointer to array of (max_ping / ping_step) 'float' entries
	long	ptime_total;			// Total played time of all players over all games
	struct	playt *playt_histo;		// Histo of total game time (seconds) vs time (e.g. hour of week)
	float	jitterwidth;			// N/100, where jitter bounds encompass N% of all ping samples

	// Heads of various important tables/lists used and/or
	// contructed during log file analysis

	struct hostaddrlist *dnslist;		// Official list of ipaddr:domainname mappings
	struct hostaddrlist *fragmentlist;	// List of 'home domains'
	struct hostaddrlist *limitlist;		// List of 'valid_domains' from which % of home domains
										//	should be calculated.
	struct asnumlist *asnumbers;		// List of AS numbers and ASCII descriptions
	int asnum_cnt;						// How many entries in asnumbers list

	struct seen_cmdtoken *cmdlist;		// List of command tokens read from logfile(s)
	struct seen_games *gamelist;		// List of all games read in from logfiles, from which
										// hangs player activities, kills, etc...
	struct seen_client *clientlist;		// Aggregate list of per-client statistics across all games
	int clientlist_cnt;					// Number of entries in clientlist
};

//
// For the global histo of played time per hour of the week,
// we keep the total time accumulated by clients during games
// in each one hour segment of the week, and also count the
// number of games played during that hour of the week.
//
struct playt {
	long playtime;
	int gamecnt;
};

// Collect a list of seen "commands" (first token after
// the timestamp in each log file line).

struct seen_cmdtoken {
	char cmdtoken[TOKENLEN];	// Token
	int tstamp;					// Timestamp of first occurrence,
								// seconds since beginning of log file
	int numseen;				// How many times seen.
	struct seen_cmdtoken *next;	// Next token in list
};

// List of "say" commands (per-game as a separate linked list)

struct seen_say {
	char *saytext;				// The actual text "said" by player
	int tstamp;					// When was this line said
	struct seen_player *who;	// Who said it
	struct seen_say *next;		// next "say" in the list
	struct seen_say *prev;		// previous "say" in the list
};

// List of join/disconnect events for a single player
// (head of list is most recent event). If 'in_ts' is non-zero
// and 'out_ts' is -1 then the player is currently active.
// 'playmode' specifies if playing or spectating.

struct player_inout {
	int	in_ts;		// Time stamp when player joined (-1 indicates preemptive
					// creation of inout entry, we're expecting the 'real'
					// ClientBegin soon)
	int out_ts;		// When player disconnected (-1 if still active)
	int frags;		// Frags achieved before leaving
	int playmode;	// 0 free play, 1 RED team, 2 BLUE team, 3 SPECTATOR
	struct player_inout *next; // Next entry
};

// History of kills (two lists hangs off each player, one for kills
// by that player, one for deaths caused by other players)

struct player_kill {
	struct seen_player *who;	// In a death list, NULL means <world> did it
	int how;					// Numeric code indicating mode of death
	int count;					// Number of times killed by this method
	struct player_kill *next;
};

// We hang a list of these entries off the per-game seen_player
// list to track any instances of where a single client changes
// its ipaddr:port *during* a single game

struct cid_ipaddr {
	struct in_addr iaddr;		// IP address in network byte order
	int port;					// UDP port number
	int resolved;				// 1 once name resolution has been attempted, 0 otherwise
	char *name;					// Pointer to ASCII domain name for this IP addr (or NULL)
								// If resolved==1 and name==NULL then the addr is unresolvable
								// (NB. Points to the associated hostaddrlist->name field)
	struct hostaddrlist *haddr;	// Pointer to hostaddrlist entry containing resolved domain
								// name and ASnumber information for this ipaddr (this is
								// somewhat redundant with the 'name' field.... 060801)
	struct cid_ipaddr *next;	// Next entry in the list.
};

// List of player_ping hangs off each seen_player entry, collecting
// in-game ping samples of suitably instrumented servers that log
// "ClientPing:" lines on a regular basis. (Not applicable for gmod1.0
// or later servers.)

struct player_ping {
	int ping;					// Ping time sampled by "ClientPing:"
	long tstamp;				// Absolute time at which this ping was sampled
	struct player_ping *next;
};

// List of player_cphisto hangs off each seen_player entry, collecting
// in-game ping histogram samples of suitably instrumented servers that log
// "CPhisto2:" lines on a regular basis.

struct player_cphisto {
	long tstamp;				// Absolute time at which this ping was sampled
	char *cphisto;				// Histogram string returned by "CPhisto:" line
	int low;					// Lowest ping value in the histo string
	int hi;						// Highest ping value in the histo string
	int lerr;
	int herr;
	int interval;				// Number of milliseconds covered by this CPhisto
	struct player_cphisto *next;
};


// List of player_item hangs off each seen_player entry, one
// entry for each type of item collected, containing a count
// of the number of times each particular item was used.

struct player_item {
	char *itemname;				// ASCII name of item as reported in "Item: N <item>" lines
	int itemcnt;				// Number of times this item has been collected
	struct player_item *next;
};

// List of players seen in this game, hanging off this are lists
// for when player joins/reconnects, kills, and deaths
// Players are uniquely identifed by their ascii playername and ipaddr

struct seen_player {
	char playername[TOKENLEN];	// Ascii player name.
	struct cid_ipaddr *c_addr;	// List of one or more ipaddr:ports this player used
	int numseen;				// Number of joins/leaves/reconnects during game
								// (but not Play/Spectate mode changes)
	int namechanged;			// Number of name changes while connected
	int frags;					// Number of frags reported at end of game (-1 if left before end)
	int ping;					// Ping time reported at end of game
	int totaltime;				// Total time this player was logged in on this map
	struct player_inout *inout; // List of start and stop times for player.
	struct player_kill *kill;	// List of kills (who, how, number)
	struct player_kill *death;	// List of deaths (by who, how, number)
	struct player_ping *pinglist;// List of ping values accumulated during this game
								// from "ClientPing:" log entries
	int pingcnt;				// Number of entries in the pinglist list
	struct player_cphisto *cph;	// List of ping histograms accumulated during this game
								// from "CPhisto:" log entries
	int cphcnt;					// Number of entries in the ping histogram list
	int medianping;				// Median ping value of pinglist or cph entries (calculated
								// during "shutdown" end of game processing).
	struct player_item *items;	// List of <item, number of times picked up>
	int itemcnt;				// Number of items picked up during game
	int fragrank;				// Rank based on frag count, 1 being top, 2 is 2nd, etc
	int pingrank;				// Rank based on median ping, 1 being top, 2 is 2nd, etc
	struct seen_player *next;	// Next token in list, or NULL
};

// List of 'current' client IDs with pointers to matching player
// info. (Player info is constant over each game, but since players
// can join and rejoin, with their client ID numbers changing in
// the meantime I need to keep separate lists.)

struct client_id {
	int c_id;					// Numeric ID assigned by server to this player
	int s_time;					// Time of most recent state change.
	int state;					// State: Connect, Changeinfo, Begin
#define C_CONN	0				// Server is aware of the new client
#define C_INFO	1				// Client registers its name, player model, etc
#define C_BEGIN	2				// Client is now "in the game"
	int playmode;				// 0 free play, 1 RED team, 2 BLUE team, 3 SPECTATOR
								// (from most recent ClientUserinfoChanged event)
	struct seen_player *c_info;	// Pointer to seen_player stats for this player
								// (defined by Playername and IPaddr:port).
								// This pointer is NULL when client_id is in C_CONN state 
	struct in_addr iaddr;		// Client's IP address in network byte order
	int port;					// Client's UDP port number
	int is_bot;					// 1 if no ipaddress (probably a bot), otherwise 0
	struct client_id *next;
};

// Linked list of games played, new entry for each new game,
//	collecting game stats per game (number of players, when
//	they joined, left, etc)

struct seen_games {
	char mapname[TOKENLEN];		// Ascii name of map being played
	int t_start;				// Time when game started, in seconds relative to t_base
	int t_end;					// Time when game ended, in seconds relative to t_base
	long t_base;				// Number of seconds since midnight (12am) Jan 1st, 1970
	int endstate;				// Reason for ending. */
#define END_FRAGLIMIT	0
#define END_TIMELIMIT	1
#define END_UNKNOWN		2
	struct client_id *c_now;	// List of client ids 'currently'
								//	active, with pointers to the matching
								//	playername info and other stats.
	struct seen_player *c_all;	// List of all the players (by name) who
								//	have been seen at any time in this game.
	int max_p_name;				// Number of player entries in c_all at end of game
	int max_c_id;				// Largest client ID seen in game (equals the
								// maximum concurrent num players any one time)
	int ranktotal;				// Total number of seen_player entries ranked (after
								// weeding out entries who were essentially idle)
	struct seen_say *say;		// List of "say" statements collected for this
								// game

	struct seen_games *next;	// Next entry (previous game)
	struct seen_games *prev;	// Prev entry (more recent game)
};



// player_ipaddr represents entries in a
// standalone linked list hanging off each seen_client entry,
// containing a unique ipaddr:port combination associated with that
// client at some point in the period covered by the logfile(s).

struct player_ipaddr {
	struct in_addr iaddr;		// IP address in network byte order
	int port;					// UDP port number
	char *fqdn;					// Resolved domain name (or NULL if resolution failed)
	struct hostaddrlist *haddr;	// Pointer to hostaddr/ASnum information for this ipaddr
	struct player_ipaddr *next;	// Next pointer in list (NULL for end of list)
};


// The seen_client structure assumes that clients are determined
// (unique <playername, suffix> tuples) as the logfiles are being
// read.
//
// NOTE 061901: The pinglist is built from copies of the linked list hanging off
// the per-game seen_player entry. This is generated from ClientPing logfile
// entries, which have been superseded by CPhisto2 as of gmod1.0
//
// NOTE 092801: Currently the seen_client list doesn't capture the
// CPhisto2 ping histograms generated by gmod1.0 or later.

struct seen_client {
	char *pname;					// The ascii playername
	int unnamedplayer;				// Flag 1 if "UnnamedPlayer", or 0 otherwise
	struct player_ipaddr *iplist;	// List of one (or possibly more) IP addr:port pairs
									// (the addr will be fixed, only the port varies)
	char *suffix;					// ASCII domain suffix uniquely associated with
									// this playername's IP address (or a fake suffix, for
									// unresolvable ip addresses)
	int numip;						// # of ipaddr:port pairs from which this name seen
	int numseen;					// How many games this client was seen in
	int	totaltime;					// Total number of seconds this player has played
	int unresolved;					// 1 if this client is due to unresolvable ipaddr(s) with
									// fake suffix in one of the ".unresolved" meta-domains
	struct player_ping *pinglist;	// List of ClientPing values accumulated over all games
	int pingcnt;					// Total number of entries in the pinglist list
	struct seen_client *next;		// Next name in list, or NULL
};

// dname_list structure is used to build an inverted tree of DNS
// names, each level of the tree is a linear list of entries
// corresponding to names at a certain depth in the domain name.
// Hanging off each level are deeper levels representing the
// next depth in the domain name. E.g. top level is ".com", ".net",
// or ".au". 2nd level contains ".home.com", ".com.au", etc... and
// so on.


struct dname_list {
	char *nfrag;				// Ascii component of domain name
	int count;					// Number of times this component seen
	int leafnum;				// Number of leaf nodes to this component
	struct dname_list *leaf;	// Points to list of subdomains
	struct dname_list *next;	// Points to next domain name fragment at this level
};




// Local DNS reverse lookup and AS number lookup table. This
// may be built from information drawn in from local files and/or
// resolved through gethostbyaddr() and 'whois' queries.

struct hostaddrlist {
	unsigned int ipaddr;		// In network byte order
	char *name;					// ASCII domain name (or NULL if unresolved/unresolvable)
	char trieddns;				// Initially zero, set to 1 if a gethostbyaddr() has already
								// been performed. Used to minimize repeat attempts to
								// gethostbyaddr() an address with no mapping
	struct asnumlist *asnum;	// Associated AS number information, or NULL if unresolved
	char triedasnum;			// Initially zero, set to 1 if a 'whois' has already
								// been performed. Used to minimize repeat attempts to
								// 'whois' an address with no mapping
	int multirec;				// n >= 1 when multiple route records were returned by whois
	int multiasnum;				// n >= 1 when different AS numbers where returned by whois
	struct hostaddrlist *next;
};

// Local version of a RADB 'whois' information:
//
// AS num
// Prefix/prefixlen
// ASCII descriptive text
//
// The AS number information in every hostaddrlist entry points to
// one of these asnumlist entries.

struct asnumlist {
	int	asnum;					// AS number (or 0 if unresolved yet)
	char *descr;				// ASCII description text for this AS number
	unsigned int	prefix;		// Route prefix associated with AS
	int prefix_len;				// Valid number of bits in prefix
	struct asnumlist *next;		// Next entry in AS number list
};

// asnum_tree  is used to created a sorted linked list
// of ASnumbers and the frequency with which they occur.

struct asnum_tree {
	int asnum;					// A single ASnumber
	int count;					// Number of times this ASnumber seen
	struct as_prefix_tree *pf;	// Pointer to list of prefixes under this AS num
								// and how many times they've been seen
	struct asnum_tree *next;
};

// A list of as_prefix_tree entries hangs off each asnum_tree entry,
// holding the counts of each prefix under the specific AS num

struct as_prefix_tree {
	struct asnumlist *asnum;	// Pointer to an actual asnumlist entry
	int count;					// Number of times this particular asnum/prefix/plen appeared
	struct as_prefix_tree *next;
};
